import pickle
import os


# General-purpose depickling function
# Inputs: filename (str)
# Returns various types
def _depickle(filename):
    with open(filename, 'rb') as handle:
        return pickle.load(handle)
    

# Convert string element of keys in MRC dictionary to all-caps
# Inputs: this_dict (dict)
# Returns a dict
def _make_keys_allcaps(this_dict):
    assert len(this_dict) > 0
    test_key = list(this_dict.keys())[0]
    
    if type(test_key) is tuple:
        return {(key[0].upper(), key[1]): value for key, value in this_dict.items()}
    else:
        return {key.upper():value for key, value in this_dict.items()}


# Convert SpaCy POS tags to best-match MRC POS tags
# Inputs: spacy_pos (str)
# Returns a tuple
def get_mrc_pos(spacy_pos):
    pos_mapping = {'PRON': ('U',),
                   'VERB': ('V', 'P'),
                   'DET': ('J',),
                   'NOUN': ('N',),
                   'ADP': ('R',),
                   'ADJ': ('J',),
                   'CCONJ': ('C',),
                   'AUX': ('V',),
                   'ADV': ('A',),
                   'PART': ('O',),
                   'NUM': ('J',),
                   'CONJ': ('C',),
                   'INTJ': ('I',),
                   'PROPN': ('N',),
                   'SCONJ': ('C',),
                   'X': ('O',)}
    
    return pos_mapping.get(spacy_pos, 'O')


# Generalized superclass for psycholinguistic databases
class PLDatabase:
    
    # Loads the pickled data structure
    # Inputs: pickle_filename (str)
    def __init__(self, pickle_filename):
        self._dict = _make_keys_allcaps(_depickle(pickle_filename))
    
    # Indicates whether a given word is included in the database
    # Inputs: word (str)
    # Returns a bool
    def _is_word_in_db(self, word):
        return word.upper() in [key.upper() for key in self._dict]

    # Generalized property retrieval ("getter") method
    # Input: word (str), feature (str)
    # Returns various types, including None if data is unavaiable
    def _get_property_from_word(self, word, feature):
        if not self._is_word_in_db(word):
            return None
        return self._dict[word.upper()][feature]
    

# Subclass to interact (read-only) with the MRC Psycholinguistic Database
# Data provided by the University of Western Australia
# Constructing an instance will load a copy of the database into memory
class MRCDatabase(PLDatabase):
    

    # Constructor method
    # Calls the superclass constructor
    def __init__(self):
        pickle_filename = __file__[:-3] + '/mrc.pickle'
        super().__init__(pickle_filename)
        
    # Indicates whether a given word is included in the MRC database
    # Inputs: word (str), pos (str)
    # Returns a bool
    def _is_word_in_db(self, word, pos):
        return (word.upper(), pos) in [(key[0].upper(), key[1]) for key in self._dict]
        
    # MRC property retrieval ("getter") method
    # Input: word (str), pos (str), feature (str)
    # Returns various types, including None if data is unavaiable
    def _get_property_from_word(self, word, pos, feature):
        match = self._dict.get((word.upper(), pos))
        
        return match[feature] if match else None
        
    # Expand an MRC part-of-speech tag to its full written form
    # Inputs: pos_indicator (str)
    # Returns a string or None
    @staticmethod
    def _expand_partofspeech(pos_indicator):
        pos_dict = {'N': 'noun', 'J': 'adjective', 'V': 'verb', 'A': 'adverb',
                    'R': 'preposition', 'C': 'conjunction', 'U': 'pronoun',
                    'I': 'interjection', 'P': 'past participle', 'O': 'other'}

        return pos_dict.get(pos_indicator)
    
    # Indicates if a given word is a derivational variant of another in the DB
    # Inputs: word (str), pos (str)
    # Returns a bool or None
    def is_derivational_variant(self, word, pos):
        tq2 = self._get_property_from_word(word, pos, 'TYPE')

        return tq2 == 'Q' if self._is_word_in_db(word, pos) else None
    
    # Indicates if a given word is capitalized in one or more usages
    # Inputs: word (str), pos (str)
    # Returns a bool or None
    def is_capitalized(self, word, pos):
        cap = self._get_property_from_word(word, pos, 'CAP')

        return cap == 'C' if self._is_word_in_db(word, pos) else None

    # Retrieve the age of acqisition score of a given word
    # Inputs: word (str), pos (str)
    # Returns an int or None
    def get_age_of_acquisition(self, word, pos):
        return self._get_property_from_word(word, pos, 'AOA')

    # Retrieve the Brown verbal frequency of a given word
    # Inputs: word (str), pos (str)
    # Returns an int or None
    def get_brown_freq(self, word, pos):
        return self._get_property_from_word(word, pos, 'BFRQ')
    
    # Retrieve the concreteness score of a given word
    # Inputs: word (str), pos (str)
    # Returns an int or None
    def get_concreteness(self, word, pos):
        return self._get_property_from_word(word, pos, 'CNC')
    
    # Retrieve the Kucera-Francis number of categories of a given word
    # Inputs: word (str), pos (str)
    # Returns an int or None
    def get_kf_num_categories(self, word, pos):
        return self._get_property_from_word(word, pos, 'KFCAT')
    
    # Retrieve the Kucera-Francis number of samples of a given word
    # Inputs: word (str), pos (str)
    # Returns an int or None
    def get_kf_num_samples(self, word, pos):
        return self._get_property_from_word(word, pos, 'KFSMP')
    
    # Retrieve the Kucera-Francis written frequency of a given word
    # Inputs: word (str), pos (str)
    # Returns an int or None
    def get_kf_freq(self, word, pos):
        return self._get_property_from_word(word, pos, 'KFFRQ')

    # Retrieve the familiarity rating of a given word
    # Inputs: word (str), pos (str)
    # Returns an int or None
    def get_familiarity(self, word, pos):
        return self._get_property_from_word(word, pos, 'FAM')
    
    # Retrieve the common part-of-speech of a given word
    # Inputs: word (str), pos (str)
    # Returns an int or None
    def get_common_partofspeech(self, word, pos):
        cpos = self._get_property_from_word(word, pos, 'CPOS')
        
        return MRCDatabase._expand_partofspeech(cpos)
    
    # Retrieve the imageability rating of a given word
    # Inputs: word (str), pos (str)
    # Returns an int or None
    def get_imageability(self, word, pos):
        return self._get_property_from_word(word, pos, 'IMG')

    # Retrieve the pronunciation variability of a given word
    # Inputs: word (str), pos (str)
    # Returns an int or None
    def get_pronunciation_var(self, word, pos):
        varp = self._get_property_from_word(word, pos, 'VARP')
        varp_dict = {'O': 'different stress patterns', 
                     'B': 'different phonology'}
        
        return varp_dict.get(varp)
    
    # Retrieve the meaningfulness score (using specified norms) of a given word
    # Inputs: word (str), pos (str), norms (str)
    # Returns an int or None
    def get_meaningfulness(self, word, pos, norms):
        available_norms = ['colorado', 'paivio']
        norms_map = {available_norms[0]: 'CMEAN', available_norms[1]: 'PMEAN'}
        
        if not norms in available_norms:
            raise ValueError('Invalid value for \'norms\'')
        
        return self._get_property_from_word(word, pos, norms_map[norms])
    
    # Retrieve the meaningfulness score (using Colorado norms) of a given word
    # Inputs: word (str), pos (str)
    # Returns an int or None
    def get_colorado_meaningfulness(self, word, pos):
        return self.get_meaningfulness(word, pos, 'colorado')
        
    # Retrieve the meaningfulness score (using Paivio norms) of a given word
    # Inputs: word (str), pos (str)
    # Returns an int or None
    def get_paivio_meaningfulness(self, word, pos):
        return self.get_meaningfulness(word, pos, 'paivio')

    # Retrieve the comprehensive syntactic category of a given word
    # Inputs: word (str), pos (str)
    # Returns a str or None
    def get_comprehensive_syntactic_category(self, word, pos):
        csyn = self._get_property_from_word(word, pos, 'CSYN')

        return MRCDatabase._expand_partofspeech(csyn)

    # Retrieve the number of phonemes of a given word
    # Inputs: word (str), pos (str)
    # Returns an int or None
    def get_num_phonemes(self, word, pos):
        return self._get_property_from_word(word, pos, 'NPHN')

    # Retrieve the morphemic status of a given word
    # Inputs: word (str), pos (str)
    # Returns a str or None
    def get_morphemic_status(self, word, pos):
        mrph = self._get_property_from_word(word, pos, 'MRPH')
        mrph_dict = {'S': 'suffix', 'P': 'prefix', 'H': 'hyphenated',
                     'A': 'abbreviation', 'T': 'multi-word phrasal unit'}

        return mrph_dict.get(mrph)
    
    # Retrieve the contextual status of a given word
    # Inputs: word (str), pos (str)
    # Returns a str or None
    def get_contextual_status(self, word, pos):
        mrph = self._get_property_from_word(word, pos, 'STAT')
        mrph_dict = {'$': 'specialized', 'A': 'archaic', 'C': 'capital', 
                     'D': 'dialect', 'E': 'nonsense', 'F': 'foreign/alien', 
                     'H': 'rhetorical', 'N': 'erroneous', 'O': 'obsolete', 
                     'P': 'poetical', 'Q': 'colloquial', 'R': 'rare', 
                     'S': 'standard', 'W': 'nonce word'}

        return mrph_dict.get(mrph)

    # Retrieve the number of syllables of a given word
    # Inputs: word (str), pos (str)
    # Returns an int or None
    def get_num_syllables(self, word, pos):
        return self._get_property_from_word(word, pos, 'NSYL')
    
    # Retrieve the number of letters of a given word
    # Inputs: word (str), pos (str)
    # Returns an int or None
    def get_num_letters(self, word, pos):
        return self._get_property_from_word(word, pos, 'NLET')
    
    # Retrieve the Thorndike-Lorge written frequency count of a given word
    # Inputs: word (str), pos (str)
    # Returns an int or None
    def get_tl_freq(self, word, pos):
        return self._get_property_from_word(word, pos, 'T-LFRQ')
    
    # Retrieve the plural type of a given word
    # Inputs: word (str), pos (str)
    # Returns a str or None
    def get_plural_type(self, word, pos):
        plur = self._get_property_from_word(word, pos, 'PLUR')
        plur_dict = {'Z': 'plural', 'B': 'both', 'Y': 'singular', 
                     'P': 'plural but acts as singular', 'N': 'no plural'}

        return plur_dict.get(plur)


# Subclass to interact (read-only) with the SUBTLEXus Database
# Data provided by Ghent University
# Constructing an instance will load a copy of the database into memory
class SubtlexDatabase(PLDatabase):
    

    # Constructor method
    # Calls the superclass constructor
    def __init__(self):
        pickle_filename = pickle_filename = __file__[:-3] + '/subtl.pickle'
        super().__init__(pickle_filename) 
    
    # Generalized SUBTL score getter method
    # Inputs: word (str), std_feature (str), log_feature (str), log_scale (bool)
    # Returns a float or None
    def _get_subtl_score(self, word, std_feature, log_feature, log_scale):
        feature_key = std_feature if not log_scale else log_feature
        score = self._get_property_from_word(word, feature_key)
        
        return score if not score else float(score)
        
    # Retrieve the SUBTLEX word form frequency of a given word
    # This metric is the frequency of the word per million words
    # Inputs: word (str), log_scale (bool)
    # Returns a float or None
    def get_subtl_wf(self, word, log_scale=False):
        return self._get_subtl_score(word, 'SUBTLWF', 'Lg10WF', log_scale)
    
    # Retrieve the SUBTLEX contextual diversity of a given word
    # This metric is the percentage of indexed films in which the word occurs
    # Inputs: word (str), log_scale (bool)
    # Returns a float or None
    def get_subtl_cd(self, word, log_scale=False):
        return self._get_subtl_score(word, 'SUBTLCD', 'Lg10CD', log_scale)



# Built-in imports
import re
import statistics as stats
import string
import logging
import os

# Third-party imports
import enchant
import numpy as np
import pandas as pd
import spacy
import textstat
from empath import Empath

import nltk
from nltk.corpus import stopwords
from nltk.tokenize import sent_tokenize, word_tokenize

from scipy.spatial import distance
from textblob import TextBlob, Word

from sklearn.feature_extraction.text import TfidfVectorizer

# Homemade imports
from wonderlic_nlp.psycholinguistics import *

class WonderlicNLP:
    def __init__(self, 
        enchant_lang = 'en_US', 
        spacy_model = 'en_core_web_sm'):

        # Global constants
        self.ENCHANT_LANG = enchant_lang
        self.SPACY_MODEL = spacy_model

        # Reusable objects
        self.__lexicon = Empath()
        self.__mrcdb = MRCDatabase()
        self.__subdb = SubtlexDatabase()
        self.__tfidf = TfidfVectorizer()
        self.__tfidf_fit = False

        try:
            self.__nlp = spacy.load(self.SPACY_MODEL)

        except Exception as e:
            logging.warning("Spacy model '{}' not installed. Installing now.".format(self.SPACY_MODEL))
            os.system('python -m spacy download {}'.format(self.SPACY_MODEL))
            self.__nlp = spacy.load(self.SPACY_MODEL)

        self.__enchant_dict = enchant.Dict(self.ENCHANT_LANG)


    def __merge(self, dictionaries):
        first = dictionaries[0]
        
        for next_ in dictionaries[1:]:
            first.update(next_)

        return first

    def analyze(self, text):
        return self.__merge([{
                'general':
                    self.__merge([self.get_num_words(text),
                    self.get_num_sentences(text),
                    self.conduct_sentiment_anaylsis(text)]),
                'mrc': self.get_mrc_features(text),
                'topic_signals': self.get_topic_signals(text),
                'punct': self.get_punct_features(text)
                },
                self.get_reading_level(text),
                self.get_pos_bag(text)])

    # `get_subtlex_from_word`: get subtlex word form frequency and contextual diversity from word
    # - Inputs: word (str)
    # - Returns dict
    def get_subtlex_from_word(self, word):
        return {'Word Form Frequency': self.__subdb.get_subtl_wf(word),
                'Contextual Diversity': self.__subdb.get_subtl_cd(word)}


    # `tfidf_fit_transform`: Learn vocabulary and idf, return document-term matrix.
    # - Inputs: list of documents (str)
    # - Returns document-term matrix (numpy aray)
    def tfidf_fit_transform(self, list_of_text):
        self.__tfidf_fit = True
        return self.__tfidf.fit_transform(list_of_text)


    # `tfidf_transform`: Transform documents to document-term matrix.
    # - Inputs: list of documents (str)
    # - Returns document-term matrix (numpy aray)
    def tfidf_transform(self, list_of_text):
        if self.__tfidf_fit:
            return self.__tfidf.transform(list_of_text)
        else:
            logging.error('The TF-IDF vectorizer is not fitted')


    # `correct_spelling`: makes a best attempt to correct spelling in a given string of text, and returns count of misspelled words
    # - Inputs: text (str)
    # - Returns str, int

    def correct_spelling(self, text):
        doc = self.strip_special_whitespace(text)
        doc = self.strip_punctuation(doc)
        words = doc.split()
        quick_fixes = {word: self.__reduce_lengthening(word) for word in words
                       if len(self.__reduce_lengthening(word)) != len(word)}
        for word, fix in quick_fixes.items():
            text = text.replace(word, fix)

        doc = TextBlob(text)

        return str(doc.correct())

    # `get_num_words`: count the number of words in a text document
    # - Inputs: document (str)
    # - Returns dict

    def get_num_words(self, document):
        return {'num_words': len(self.strip_punctuation(document).split())}


    # `get_num_sentences`: count the number of sentences in a text document
    # - Inputs: document (str)
    # - Returns dict

    def get_num_sentences(self, document):
        return {'num_sentences': len(sent_tokenize(document))}

    # `get_avg_sentence_len`: calculate the average number of words per sentence in a text document
    # - Inputs: document (str)
    # - Returns dict

    def get_avg_sentence_len(self, document):
        num_words = self.get_num_words(document)['num_words']
        num_sentences = self.get_num_sentences(document)['num_sentences']
        return {'avg_sent_len': num_words / num_sentences}

    # `get_topic_signals`: uses a topic engine (e.g. Empath) to extract a count of topic occurences in text
    # - Inputs: document (str), engine (str)
    # - Returns dict

    def get_topic_signals(self, document, engine='empath'):
        if engine == 'empath':
            topic_counts = self.__lexicon.analyze(document, normalize=False)
        else:
            raise ValueError('engine must be \'empath\'')

        return {engine + '_' + topic: value for topic, value in topic_counts.items()}

    # `get_mrc_features`: collects and aggregates psycholinguistic features from the MRC database for words in the provided document
    # - Inputs: document (str)
    # - Returns dict

    def get_mrc_features(self, document):
        accessors = self.__get_mrc_accessors(self.__mrcdb)
        feature_tracker = {feature: [] for feature in accessors}

        doc = self.strip_special_whitespace(document)
        doc = self.strip_punctuation(doc)
        doc = self.__nlp(doc)
        for token in doc:
            if token.pos_ in ('SPACE', 'SYM', 'EOL'):
                continue
            word = token.text
            pos_tuple = get_mrc_pos(token.pos_)
            for feature in accessors:
                for pos in pos_tuple:
                    feature_value = accessors[feature](word, pos)
                    if feature_value:
                        break
                if feature_value:
                    # if type(feature_value) is str:
                    #     print(feature)
                    #     print(feature_value)
                    feature_tracker[feature].append(feature_value)

        mrc_analysis = {}
        for feature, tracker in feature_tracker.items():
            if len(tracker) > 0:
                mrc_analysis[feature] = stats.mean(tracker)
            else:
                mrc_analysis[feature] = 0 # Temporary for AOA None due to sparsity error.

        return mrc_analysis

    # `__get_mrc_accessors`: retrieves a mapping of MRC features to their getter methods
    # - Inputs: mrc_database (MRCDatabase obj)
    # - Returns dict

    def __get_mrc_accessors(self, mrc_database):
        return {'Nlet': mrc_database.get_num_letters,
                'Nphon': mrc_database.get_num_phonemes,
                'Nsyl': mrc_database.get_num_syllables,
                'K-F-freq': mrc_database.get_kf_freq,
                'K-F-ncats': mrc_database.get_kf_num_categories,
                'K-F-nsamp': mrc_database.get_kf_num_samples,
                'T-L-freq': mrc_database.get_tl_freq,
                'Brown-freq': mrc_database.get_brown_freq,
                'Fam': mrc_database.get_familiarity,
                'Conc': mrc_database.get_concreteness,
                'Imag': mrc_database.get_imageability,
                'Meanc': mrc_database.get_colorado_meaningfulness,
                'Meanp': mrc_database.get_paivio_meaningfulness,
                'AOA': mrc_database.get_age_of_acquisition}


    # ## Pre-processing & helper functions

    # `strip_special_whitespace`: removes all '\t' and '\n' escape characters from the provided text
    # - Inputs: text (str)
    # - Returns str

    def strip_special_whitespace(self, text):
        return text.translate(str.maketrans('', '', '\n\t'))


    # `strip_punctuation`: removes all punctuation from the provided text
    # - Inputs: text (str)
    # - Returns str

    def strip_punctuation(self, text):
        return text.translate(str.maketrans('', '', '!"#$%&()*+,-./:;<=>?@[\\]^_`{|}~'))


    # `remove_stopwords`: strips a string of all word tokens matching a set of stop words
    # - Inputs: text (str)
    # - Returns str

    def remove_stopwords(self, text):
        stops = set(stopwords.words('english'))
        word_tokens = word_tokenize(text)
        filtered_tokens = [word for word in word_tokens if not word in stops]

        return ' '.join(filtered_tokens)


    # `lemmatize`: converts words in a text string to their meaningful base forms
    # - Inputs: text (str)
    # - Returns str

    def lemmatize(self, text):
        doc = self.__nlp(text)
        lemma_tokens = [token.lemma_ for token in doc]

        return ' '.join(lemma_tokens)


    # `__average_over_sentences`: converts sentence-level, dict-returning feature extractions to document-level
    # - Inputs: document (str), sentence_function (function), remove_punctuation (bool)
    # - Returns dict

    def __average_over_sentences(self, document, sentence_function, remove_punctuation=False):
        sentences = sent_tokenize(document)
        if remove_punctuation:
            sentences = [strip_punctuation(sentence) for sentence in sentences]

        aggregator = None
        sent_num = 0
        for sentence in sentences:
            sent_num += 1
            sentence_features = sentence_function(sentence)
            if not aggregator:
                aggregator = {key: [value] if value else []
                              for key, value in sentence_features.items()}
            else:
                for key in sentence_features:
                    if sentence_features[key]:
                        aggregator[key] = aggregator.get(key, []) + [sentence_features[key]]

        return {feature: stats.mean(tracker) if len(tracker) > 0 else None
                for feature, tracker in aggregator.items()}


    # `__reduce_lengthening`: forces any group of a single repeated letter down to no more than two letters (e.g. "bubbble" -> "bubble")
    # - Inputs: text (str)
    # - Returns str

    def __reduce_lengthening(self, text):
        return re.compile(r"(.)\1{2,}").sub(r"\1\1", text)


    # `conduct_sentiment_analysis`: computes the average sentence polarity metric across a text document
    # - Inputs: document (str)
    # - Returns dict

    def conduct_sentiment_anaylsis(self, document):
        blob = TextBlob(document)
        polarities = [sentence.sentiment.polarity for sentence in blob.sentences]

        return {'average_polarity': stats.mean(polarities)}

    # `get_pos_bag`: create a "bag of tags" for the parts-of-speech tagged in the document
    # - Inputs: document (str)
    # - Returns dict

    def get_pos_bag(self, document):
        bag_of_pos = {}
        doc = self.__nlp(document)
        for token in doc:
            bag_of_pos[token.pos_] = bag_of_pos.get(token.pos_, 0) + 1

        return {'bag_of_pos': bag_of_pos}


    # `get_bag_of_words`: create a key-value mapped bag of words
    # - Inputs: document (str)
    # - Returns dict

    def get_bag_of_words(self, document):
        doc = self.strip_special_whitespace(document)
        doc = self.strip_punctuation(doc)
        bag_of_words = {}
        for word in word_tokenize(doc):
            bag_of_words[word.lower()] = bag_of_words.get(word, 0) + 1

        return {'bag_of_words': bag_of_words}


    # `get_punct_features`: count punctuation marks used throughout the document
    # - Inputs: document (str)
    # - Returns dict

    def get_punct_features(self, document):
        puncts = [char for char in document if char in string.punctuation]
        punct_features = {'Period': puncts.count('.'),
                          'Comma': puncts.count(','),
                          'Colon': puncts.count(':'),
                          'Semic': puncts.count(';'),
                          'Qmark': puncts.count('?'),
                          'Exclam': puncts.count('!'),
                          'Dash': puncts.count('-'),
                          'Quote': puncts.count('"'),
                          'Apostro': puncts.count('\''),
                          'Parenth': puncts.count('(') + puncts.count(')')}
        punct_features['Otherp'] = len(puncts) - sum(punct_features.values())
        punct_features['Allpct'] = len(puncts)

        return {'punct': punct_features}


    # `get_reading_level`: determine the approximate reading level of the document based on a consensus of the Flesch Reading Ease formula, the Flesch-Kincaid Grade Level, the Fog Scale, the SMOG Index, the Automated Readability Index, the Coleman-Liau Index, the Linsear Write Formula, and the Dale-Chall Readability Score
    # - Inputs: document (str)
    # - Returns dict

    def get_reading_level(self, document):
        return {'reading_level': textstat.text_standard(document, float_output=True)}


    # `do_spell_check`: runs a spell checker on the text and returns a count and ratio of misspelled words
    # - Inputs: document (str)
    # - Returns dict

    def do_spell_check(self, document):
        doc = self.strip_special_whitespace(document)
        doc = self.strip_punctuation(doc)
        words = doc.split()
    
        num_misspelled = 0
        misspelled_words = []
        for word in words:
            if not self.__enchant_dict.check(word):
                misspelled_words.append(word)
                num_misspelled += 1

        return {'count_misspelled': num_misspelled, 
                'ratio_misspelled': num_misspelled / len(words),
                'misspelled_words': misspelled_words}



from setuptools import setup
setup(
  name = 'wonderlic_nlp',
  packages = ['wonderlic_nlp'],
  version = '0.1.1',
  license='gpl-3.0',
  description = 'General-purpose NLP toolkit for accessing popular psycholinguistics databases.',
  author = 'Ross Piper, Abdallah Aboelela',
  author_email = 'ross.piper@wonderlic.com, abdallah.aboelela@wonderlic.com',
  url = 'https://github.com/Wonderlic-AI/wonderlic_nlp',
  download_url = 'https://github.com/Wonderlic-AI/wonderlic_nlp/archive/v_0_1_1.tar.gz',
  keywords = ['NLP', 'Python', 'Natural', 'Language', 'Processing', 'English', 'Python', 'tfidf', 'MRC', 'SUBTLEX', 'SUBTLEXus', 'Psycholinguistics'],
  install_requires=[            # I get to this in a second
          'pandas',
          'scipy',
          'textblob',
          'spacy',
          'nltk',
          'empath',
          'numpy',
          'pyenchant',
          'sklearn',
          'textstat'
      ],
  include_package_data = True,
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'Programming Language :: Python :: 3.7',
  ],
)